package methods;

import algorithms.Classifier1NN;
import algorithms.DistanceTemplatesBaseline;
import algorithms.DistanceVectorsL1;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.Feature;
import objects.Motion;
import objects.MotionJointCoordinates;
import objects.Parameter;

public class MethodAli extends Method {

    public MethodAli() {
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return Executor.loadMotionJointCoordinates(fileAMC);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {
        Classifier1NN classifier = new Classifier1NN();
        DistanceTemplatesBaseline distanceTemplates = new DistanceTemplatesBaseline();
        distanceTemplates.setDistanceVectors(new DistanceVectorsL1());
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) { // mean of the hip-knee-ankle triangle areas
        MotionJointCoordinates motionMJC = (MotionJointCoordinates) motion;
        List<Feature> features = new ArrayList();
        features.add(new Feature(new Parameter("left", motionMJC.extractTriangleAreaFeature("lhipjoint", "lfemur", "ltibia").getMean())));
        features.add(new Feature(new Parameter("right", motionMJC.extractTriangleAreaFeature("rhipjoint", "rfemur", "rtibia").getMean())));
        return features;
    }
}
