package methods;

import algorithms.Classifier1NN;
import algorithms.DistanceTemplatesDTW;
import algorithms.DistanceVectorsL1;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.Feature;
import objects.Motion;
import objects.MotionJointCoordinates;

public class MethodGavrilova extends Method {

    public MethodGavrilova() {
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return Executor.loadMotionJointCoordinates(fileAMC);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {
        Classifier1NN classifier = new Classifier1NN();
        DistanceTemplatesDTW distanceTemplates = new DistanceTemplatesDTW();
        distanceTemplates.setDistanceVectors(new DistanceVectorsL1());
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) { // JRD+JRA
        MotionJointCoordinates motionMJC = (MotionJointCoordinates) motion;
        List<Feature> features = new ArrayList();
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("lowerneck", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("lclavicle", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("rclavicle", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("thorax", "rtibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "lfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "rfemur"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "ltibia"));
        features.add(motionMJC.extractInterJointDistanceFeature("root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("lclavicle", "root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("rclavicle", "root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "lfemur"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "rfemur"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("lhipjoint", "root", "rtibia"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "ltibia"));
        features.add(motionMJC.extractJointAngleFeature("rhipjoint", "root", "rtibia"));
        return features;
    }
}
