package methods;

import algorithms.Classifier1NN;
import algorithms.DistanceTemplatesDTW;
import algorithms.DistanceVectors;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.Axis;
import objects.Feature;
import objects.Motion;
import objects.MotionJointCoordinates;

public class MethodKrzeszowski extends Method {

    public MethodKrzeszowski() {
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return Executor.loadMotionJointCoordinates(fileAMC);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {
        Classifier1NN classifier = new Classifier1NN();
        DistanceTemplatesDTW distanceTemplates = new DistanceTemplatesDTW();
        distanceTemplates.setDistanceVectors(new DistanceVectorsKrzeszowski());
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) { // 8 bones * 3 angles + height + ankledistance
        MotionJointCoordinates motionMJC = (MotionJointCoordinates) motion;
        List<Feature> features = new ArrayList();
        for (Axis axis : getAxes()) {
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "ltibia", "lfemur"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "rtibia", "rfemur"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "lfemur", "lhipjoint"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "rfemur", "rhipjoint"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "lwrist", "lhumerus"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "rwrist", "rhumerus"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "lhumerus", "lclavicle"));
            features.add(motionMJC.extractBoneAxisAngleFeature(axis, "rhumerus", "rclavicle"));
        }
        features.add(motionMJC.extractJointAxisCoordinateFeature(new Axis("Y"), "head"));
        features.add(motionMJC.extractInterJointDistanceFeature("ltibia", "rtibia"));
        return features;
    }

    public final class DistanceVectorsKrzeszowski extends DistanceVectors {

        public DistanceVectorsKrzeszowski() {
        }

        @Override
        public double getDistance(double[] pose1, double[] pose2) {
            double w1 = 57.3;
            double w2 = 1.7;
            int length = pose1.length;
            if (pose2.length != length) {
                System.out.println("wrong dimensions");
            }
            double rho = 0;
            int d = 0;
            while (d < length - 2) {
                double sumAngles = 0;
                for (int i = 0; i < 3; i++) {
                    sumAngles += Math.pow(180 - Math.abs(Math.abs(pose1[d + i] - pose2[d + i]) - 180), 2);
                    d++;
                }
                rho += Math.sqrt(sumAngles);
            }
            double heightAndAnkleDistanceDifference = 0;
            while (d < length) {
                heightAndAnkleDistanceDifference += Math.abs(pose1[d] - pose2[d]);
                d++;
            }
            return w1 * rho + w2 * heightAndAnkleDistanceDifference;
        }
    }
}
