package methods;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;
import algorithms.Classifier1NN;
import algorithms.DistanceTemplates;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.Feature;
import objects.Motion;
import objects.Parameter;
import objects.Template;

public class MethodKumar extends Method {

    private static final int length = 150; //?

    public MethodKumar() {
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return Executor.loadMotionJointCoordinates(fileAMC);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {
        Classifier1NN classifier = new Classifier1NN();
        DistanceTemplatesKumar distanceTemplates = new DistanceTemplatesKumar();
        distanceTemplates.setMeanMatrix(getMeanMatrix(extractTemplates(motionsLearning)));
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) {
        motion.adjust(length);
        double[][] data = getVectorJointCoordinates(motion).getArray();
        double[] vector = new double[data.length];
        for (int i = 0; i < data.length; i++) {
            vector[i] = data[i][0];
        }
        Matrix matrix = new Matrix(vector, getAxes().length * length);
        List<Feature> features = new ArrayList();
        for (int r = 0; r < matrix.getRowDimension(); r++) {
            Parameter[] parameters = new Parameter[matrix.getColumnDimension()];
            for (int c = 0; c < matrix.getColumnDimension(); c++) {
                parameters[c] = new Parameter(String.valueOf(c), matrix.get(r, c));
            }
            features.add(new Feature(parameters));
        }
        return features;
    }

    public final class DistanceTemplatesKumar extends DistanceTemplates {

        Matrix meanMatrix;

        public DistanceTemplatesKumar() {
        }

        public void setMeanMatrix(Matrix meanMatrix) {
            this.meanMatrix = meanMatrix;
        }

        @Override
        public double getDistance(Template template1, Template template2) {
            double sum = 0.0;
            double[] generalizedEigenvalues = new EigenvalueDecomposition(getCovarianceMatrix(template2).inverse().times(getCovarianceMatrix(template1))).getRealEigenvalues();
            for (int l = 0; l < generalizedEigenvalues.length; l++) {
                sum += Math.pow(Math.log(Math.abs(generalizedEigenvalues[l])), 2);
            }
            return Math.sqrt(sum);
        }

        private Matrix getCovarianceMatrix(Template template) {
            Matrix matrix = template.getMatrix();
            int rows = matrix.getRowDimension();
            int columns = matrix.getColumnDimension();
            Matrix covarianceMatrix = new Matrix(columns, columns);
            for (int r = 0; r < rows; r++) {
                Matrix d = matrix.getMatrix(r, r, 0, columns - 1);
                Matrix u = meanMatrix.getMatrix(r, r, 0, columns - 1);
                Matrix diff = d.minus(u);
                covarianceMatrix.plusEquals((diff.transpose().times(diff)).times((double) 1 / (rows - 1)));
            }
            return covarianceMatrix;
        }
    }
}
