package methods;

import Jama.EigenvalueDecomposition;
import Jama.Matrix;
import algorithms.Classifier;
import algorithms.Classifier1NN;
import algorithms.ClassifierLinear;
import algorithms.DistanceTemplatesBaseline;
import algorithms.DistanceVectorsMahalanobis;
import com.mkobos.pca_transform.PCA;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.ClassifiableList;
import objects.Feature;
import objects.Motion;
import objects.Parameter;
import objects.Sample;

public class Method_PCALDA extends Method {

    private final boolean dataType; // true = BoneRotations, false = JointCoordinates
    private final Classifier classifier = new Classifier1NN();
    private final int length = 150;

    public Method_PCALDA(boolean dataType) {
        this.dataType = dataType;
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return dataType ? Executor.loadMotionBoneRotations(fileAMC) : Executor.loadMotionJointCoordinates(fileAMC);
    }

    private Matrix getVector(Motion motion) {
        return dataType ? getVectorBoneRotations(motion) : getVectorJointCoordinates(motion);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {

        // TRANSFORMATION MATRIX
        List<Sample> samples = new ArrayList();
        for (Motion motion : motionsLearning) {
            motion.adjust(length);
            samples.add(new Sample(motion.getSubject(), motion.getId(), getVector(motion)));
        }
        Matrix meanSampleMatrix = getMeanMatrix(samples);
        int dimension = samples.get(0).getDimension();
        Matrix samplesMatrix = new Matrix(samples.size(), dimension);
        for (int d = 0; d < dimension; d++) {
            for (int s = 0; s < samples.size(); s++) {
                samplesMatrix.set(s, d, samples.get(s).getParameterValue(d, 0) - meanSampleMatrix.get(d, 0));
            }
        }
        Matrix SigmaB = getBetweenClassScatterMatrix(samples);
        Matrix SigmaW = getWithinClassScatterMatrix(samples);
        Matrix PhiPCA = new PCA(samplesMatrix).getEigenvectorsMatrix().getMatrix(0, dimension - 1, 0, new ClassifiableList(samples).splitClasses().size() - 1);
        Matrix PhiLDA = new EigenvalueDecomposition((PhiPCA.transpose().times(SigmaW).times(PhiPCA)).inverse().times(PhiPCA.transpose().times(SigmaB).times(PhiPCA))).getV();
        classifier.setTransformationMatrix(PhiPCA.times(PhiLDA));
        setClassifier(classifier);

        // DISTANCE FUNCTION
        DistanceTemplatesBaseline distanceTemplates = new DistanceTemplatesBaseline();
        DistanceVectorsMahalanobis distanceVectors = new DistanceVectorsMahalanobis();
        distanceVectors.setMatrix(getTotalScatterMatrix(extractTemplates(motionsLearning)).inverse());
        distanceTemplates.setDistanceVectors(distanceVectors);
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) {
        motion.adjust(length);
        double[][] data = getClassifier().getTransformationMatrix().transpose().times(getVector(motion)).getArray();
        List<Feature> features = new ArrayList();
        for (int d = 0; d < data.length; d++) {
            features.add(new Feature(new Parameter[]{new Parameter(String.valueOf(d), data[d][0])}));
        }
        return features;
    }
}
