package methods;

import algorithms.Classifier;
import algorithms.Classifier1NN;
import algorithms.DistanceTemplatesDTW;
import algorithms.DistanceVectors;
import algorithms.DistanceVectorsL1;
import algorithms.DistanceVectorsL2;
import executor.Executor;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import objects.Feature;
import objects.Motion;
import objects.Parameter;

public class Method_Raw extends Method {

    private final boolean dataType; // true = BoneRotations, false = JointCoordinates
    private final Classifier classifier = new Classifier1NN();

    public Method_Raw(boolean dataType) {
        this.dataType = dataType;
    }

    @Override
    public Motion loadMotion(File fileAMC) throws IOException {
        return dataType ? Executor.loadMotionBoneRotations(fileAMC) : Executor.loadMotionJointCoordinates(fileAMC);
    }

    @Override
    public void learnClassifier(List<Motion> motionsLearning) {
        DistanceTemplatesDTW distanceTemplates = new DistanceTemplatesDTW();
        distanceTemplates.setDistanceVectors(new DistanceVectors_Raw());
        classifier.setDistanceTemplates(distanceTemplates);
        setClassifier(classifier);
    }

    @Override
    public List<Feature> extractFeatures(Motion motion) {
        List<Feature> features = new ArrayList();
        if (dataType) {
            double[][] data = getTensorBoneRotations(motion);
            for (int b = 0; b < data[0].length; b++) {
                Parameter[] parameters = new Parameter[data.length];
                for (int l = 0; l < data.length; l++) {
                    parameters[l] = new Parameter(String.valueOf(l) + ":" + String.valueOf(b), data[l][b]);
                }
                features.add(new Feature(parameters));
            }
        } else {
            double[][][] data = getTensorJointCoordinates(motion);
            for (int j = 0; j < data[0].length; j++) {
                for (int a = 0; a < data[0][0].length; a++) {
                    Parameter[] parameters = new Parameter[data.length];
                    for (int l = 0; l < data.length; l++) {
                        parameters[l] = new Parameter(String.valueOf(l) + ":" + String.valueOf(j) + getAxes()[a].getLabel(), data[l][j][a]);
                    }
                    features.add(new Feature(parameters));
                }
            }
        }
        return features;
    }

    public final class DistanceVectors_Raw extends DistanceVectors {

        public DistanceVectors_Raw() {
        }

        @Override
        public double getDistance(double[] pose1, double[] pose2) {
            int length = pose1.length;
            if (length != pose2.length) {
                System.out.println("wrong dimensions");
            }
            if (dataType) {
                return new DistanceVectorsL1().getDistance(pose1, pose2);
            } else {
                int axesCount = getAxes().length;
                double[][] pose1axes = new double[length / axesCount][axesCount];
                double[][] pose2axes = new double[length / axesCount][axesCount];
                for (int d = 0; d < length; d++) {
                    pose1axes[d / axesCount][d % axesCount] = pose1[d];
                    pose2axes[d / axesCount][d % axesCount] = pose2[d];
                }
                double sum = 0;
                for (int d = 0; d < length / axesCount; d++) {
                    sum += Math.abs(new DistanceVectorsL2().getDistance(pose1axes[d], pose2axes[d]));
                }
                return sum;
            }
        }
    }
}
