package objects;

import algorithms.DistanceTemplates;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DistanceMatrix {

    private final DistanceTemplates distanceTemplates;
    private final Map<String, Double> distanceMap = new HashMap();
    private final List<String> templateCodes = new ArrayList();
    private final String separator = "#";

    public DistanceMatrix(DistanceTemplates distanceTemplates) {
        this.distanceTemplates = distanceTemplates;
    }

    private String getTemplateCode(Template template) {
        return template.getSubject() + "_" + template.getId();
    }

    private String getKey(Template template1, Template template2) {
        return getTemplateCode(template1) + separator + getTemplateCode(template2);
    }

    public void putDistance(Template template1, Template template2) {
        String key = getKey(template1, template2);
        distanceMap.put(key, distanceTemplates.getDistance(template1, template2));
        String code = key.split(separator)[0];
        if (!templateCodes.contains(code)) {
            templateCodes.add(code);
        }
    }

    public double getDistance(Template template1, Template template2) {
        return distanceMap.get(getKey(template1, template2));
    }

    public void print() {
        Collections.sort(templateCodes);
        System.out.print(",");
        for (String templateCode : templateCodes) {
            System.out.print(templateCode + ",");
        }
        System.out.print("\r\n");
        for (String templateCode1 : templateCodes) {
            System.out.print(templateCode1 + ",");
            for (String templateCode2 : templateCodes) {
                System.out.print(distanceMap.get(templateCode1 + separator + templateCode2) + ",");
            }
            System.out.print("\r\n");
        }
    }
}
