package objects;

import java.util.ArrayList;
import java.util.List;

public class Motion<T> extends Classifiable {

    private List<Frame> frames;

    public Motion(int subject, String id, List<Frame> frames) {
        super(subject, id);
        this.frames = frames;
    }

    public void setFrames(List<Frame> frames) {
        this.frames = frames;
    }

    public int getLength() {
        return frames.size();
    }

    public List<Frame> getFrames() {
        return frames;
    }

    public Frame getFrame(int index) {
        return frames.get(index);
    }

    public Motion getSubMotion(int from, int to) {
        int length = getLength();
        if (to > length) {
            System.out.println("submotion incomplete: " + to + ">" + length);
        }
        List<Frame> subFrames = new ArrayList();
        for (int l = from; l < to && l < length; l++) {
            subFrames.add(getFrame(l));
        }
        return new Motion(getSubject(), getId() + "[" + (from + 1) + "-" + (to + 1) + "]", subFrames);
    }

    public void adjust(int length) {
        List<Frame> adjustedFrames = new ArrayList();
        for (int l = 0; l < length; l++) {
            adjustedFrames.add(getFrame(Math.round((float) l * (getLength() - 1) / (length - 1))));
        }
        setFrames(adjustedFrames);
    }
}
