package objects;

import java.util.List;

public class MotionJointCoordinates extends Motion {

    public MotionJointCoordinates(int subject, String id, List<FrameJointCoordinates> frames) {
        super(subject, id, frames);
    }

    public Feature extractFeetDistanceSideSensitiveFeature() {
        Axis z = new Axis("Z");
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            FrameJointCoordinates frame = (FrameJointCoordinates) getFrame(l);
            parameters[l] = new Parameter("FeetDistanceSideSensitive", frame.extractJointAxisCoordinateParameter(z, "lfoot").getValue() - frame.extractJointAxisCoordinateParameter(z, "rfoot").getValue());
        }
        return new Feature(parameters);
    }

    public Feature extractInterJointDistanceFeature(String jointName1, String jointName2) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractInterJointDistanceParameter(jointName1, jointName2);
        }
        return new Feature(parameters);
    }

    public Feature extractJointAngleFeature(String jointName1, String jointName2, String jointName3) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractJointAngleParameter(jointName1, jointName2, jointName3);
        }
        return new Feature(parameters);
    }

    public Feature extractJointAxisCoordinateFeature(Axis axis, String jointName) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractJointAxisCoordinateParameter(axis, jointName);
        }
        return new Feature(parameters);
    }

    public Feature extractJointAxisDistanceFeature(Axis axis, String jointName1, String jointName2) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractJointAxisDistanceParameter(axis, jointName1, jointName2);
        }
        return new Feature(parameters);
    }

    public Feature extractBoneAxisAngleFeature(Axis axis, String jointName1, String jointName2) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractBoneAxisAngleParameter(axis, jointName1, jointName2);
        }
        return new Feature(parameters);
    }

    public Feature extractCentroidsDistanceFeature(String[] jointNames1, String[] jointNames2) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractCentroidsDistanceParameter(jointNames1, jointNames2);
        }
        return new Feature(parameters);
    }

    public Feature extractTriangleAreaFeature(String jointName1, String jointName2, String jointName3) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractTriangleAreaParameter(jointName1, jointName2, jointName3);
        }
        return new Feature(parameters);
    }

    public Feature extractProjectedPolygonAreaFeature(String[] jointNames, Axis projectionAxis1, Axis projectionAxis2) {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = ((FrameJointCoordinates) getFrame(l)).extractProjectedPolygonAreaParameter(jointNames, projectionAxis1, projectionAxis2);
        }
        return new Feature(parameters);
    }

    public Feature extractZeroFeature() {
        int length = getLength();
        Parameter[] parameters = new Parameter[length];
        for (int l = 0; l < length; l++) {
            parameters[l] = new Parameter("zero", 0);
        }
        return new Feature(parameters);
    }
}
